import tkinter as tk
import tkinter.filedialog as fd
import pandas as pd
import numpy as np
import pystan
import matplotlib.pyplot as plt
import seaborn as sb
import scipy.stats as ss

def askinfilename(msg = None, types = [('', '*.*')]):
    """
            入力用ファイル名の設定
    """
    rt = tk.Tk()
    rt.withdraw()
    flnm = fd.askopenfilename(title = msg, filetypes = types)
    rt.destroy()
    return flnm

def askoutfilename(msg = None, types = [('', '*.*')], deftype = '.txt'):
    """
            出力用ファイル名の設定
    """
    rt = tk.Tk()
    rt.withdraw()
    flnm = fd.asksaveasfilename(title = msg, filetypes = types,
                                           defaultextension = deftype)
    rt.destroy()
    return flnm


finnm = askinfilename('入力データExcelファイル', types = [('Excel(*.xlsx)', '*.xlsx')])
txtoutnm = askoutfilename(msg = '出力ファイル', types = [('Text (*.txt)', '*.txt')])
txtout = open(txtoutnm, 'w')
foutnm = askoutfilename(msg = '画像保存ファイル', types = [('画像 (*.png)', '*.png')],
                       deftype = '.png')

xlsx_fl = pd.ExcelFile(finnm)
data_xlsx = pd.read_excel(xlsx_fl,  header = None)
data = data_xlsx.values
print(data)
txtout.write('\n入力データファイル:\n')
txtout.write('    {0}\n'.format(finnm))

VarNameA = data[0][1]
VarNameB = data[0][2]

dataA = []
dataB = []
for i in range(1,len(data)):
    if not pd.isnull(data[i][1]):
        dataA.append(data[i][1])
    if not pd.isnull(data[i][2]):
        dataB.append(data[i][2])

txtout.write('\n{0}\n'.format(VarNameA))
txtout.write('{0}\n'.format(dataA))
txtout.write('\n{0}\n'.format(VarNameB))
txtout.write('{0}\n'.format(dataB))

StanData = {'NA': len(dataA), 'DataA': dataA,
            'NB': len(dataB), 'DataB': dataB}

sm = pystan.StanModel(file = 'IndDiffMu2Sgm.stan')
fit = sm.sampling(data = StanData, n_jobs = 1)

print(fit)

txtout.write('\n{0}\n'.format(fit))

#       日本語表記の設定
plt.rcParams['font.family'] = ['Yu Gothic', 'Noto Sans CJK JP', 'Hiragino Maru Gothic Pro']

muA_p025, muA_p25, muA_p50, muA_p75, muA_p975 = \
    np.percentile(fit['muA'], [2.5, 25, 50, 75, 97.5])
muA_mean = fit['muA'].mean()
muB_p025, muB_p25, muB_p50, muB_p75, muB_p975 = \
    np.percentile(fit['muB'], [2.5, 25, 50, 75, 97.5])
muB_mean = fit['muB'].mean()
txtout.write('\nmuA:\n')
txtout.write('    mean = {0:.3f}\n'.format(muA_mean))
txtout.write('    95%CI = [{0:.3f}, {1:.3f}]\n'.format(muA_p025, muA_p975))
txtout.write('    Q1 = {0:.3f},    Median = {1:.3f},    Q3 = {2:.3f}\n'.format(
                    muA_p25, muA_p50, muA_p75))

txtout.write('\nmuB:\n')
txtout.write('    mean = {0:.3f}\n'.format(muB_mean))
txtout.write('    95%CI = [{0:.3f}, {1:.3f}]\n'.format(muB_p025, muB_p975))
txtout.write('    Q1 = {0:.3f},    Median = {1:.3f},    Q3 = {2:.3f}\n'.format(
                    muB_p25, muB_p50, muB_p75))

plt.title('平均値の事後分布', fontsize = 20)
sb.kdeplot(fit['muA'], label = VarNameA)
sb.kdeplot(fit['muB'], label = VarNameB)
plt.yticks([])
plt.legend(fontsize = 12)
plt.savefig(foutnm[:-4] + '_muAB.png')
plt.show()

mean_mu_d = fit['mu_d'].mean()
mu_d025P, mu_d25P, mu_d50P, mu_d75P, mu_d975P = \
          np.percentile(fit['mu_d'], [2.5, 25, 50, 75, 97.5])

txtout.write('\nmu_d:\n')
txtout.write('    mean = {0:.3f}\n'.format(mean_mu_d))
txtout.write('    95%CI = [{0:.3f}, {1:.3f}]\n'.format(mu_d025P, mu_d975P))
txtout.write('    Q1 = {0:.3f},    Median = {1:.3f},    Q3 = {2:.3f}\n'.format(
                    mu_d25P, mu_d50P, mu_d75P))

sb.kdeplot(fit['mu_d'])
plt.title('平均値の差の事後分布' +
          '\n平均 = {0:.2f}   95%CI = [{1:.2f}, {2:.2f}]'.format(
              mean_mu_d, mu_d025P, mu_d975P), fontsize = 16)
plt.yticks([])
plt.xlabel('平均値({0}) - 平均値({1})'.format(VarNameA, VarNameB), fontsize = 14)
plt.savefig(foutnm[:-4] + '_mu_d.png')
plt.show()

mean_sgm_r = fit['sgm_r'].mean()
sgm_r025P, sgm_r25P, sgm_r50P, sgm_r75P, sgm_r975P = \
          np.percentile(fit['sgm_r'], [2.5, 25, 50, 75, 97.5])

txtout.write('\nsgm_r:\n')
txtout.write('    mean = {0:.3f}\n'.format(mean_sgm_r))
txtout.write('    95%CI = [{0:.3f}, {1:.3f}]\n'.format(sgm_r025P, sgm_r975P))
txtout.write('    Q1 = {0:.3f},    Median = {1:.3f},    Q3 = {2:.3f}\n'.format(
                    sgm_r25P, sgm_r50P, sgm_r75P))

plt.title(('標準偏差の比SD({0})/SD({1})の事後分布'.format(VarNameA, VarNameB)) +
          '\n平均 = {0:.3f},   95%CI = [{1:.3f}, {2:.3f}]'.format(
              mean_sgm_r, sgm_r025P, sgm_r975P), fontsize = 18)
sb.kdeplot(fit['sgm_r'])
plt.xlabel('SD({0})/SD({1})'.format(VarNameA, VarNameB), fontsize = 14)
plt.tight_layout()
plt.savefig(foutnm[:-4] + '_sgm_r.png')
plt.show()
 
sgmA_p025, sgmA_p25, sgmA_p50, sgmA_p75, sgmA_p975 = \
    np.percentile(fit['sgmA'], [2.5, 25, 50, 75, 97.5])
sgmA_mean = fit['sgmA'].mean()
sgmB_p025, sgmB_p25, sgmB_p50, sgmB_p75, sgmB_p975 = \
    np.percentile(fit['sgmB'], [2.5, 25, 50, 75, 97.5])
sgmB_mean = fit['sgmB'].mean()
txtout.write('\nsgmA:\n')
txtout.write('    mean = {0:.3f}\n'.format(sgmA_mean))
txtout.write('    95%CI = [{0:.3f}, {1:.3f}]\n'.format(sgmA_p025, sgmA_p975))
txtout.write('    Q1 = {0:.3f},    Median = {1:.3f},    Q3 = {2:.3f}\n'.format(
                    sgmA_p25, sgmA_p50, sgmA_p75))

txtout.write('\nsgmB:\n')
txtout.write('    mean = {0:.3f}\n'.format(sgmB_mean))
txtout.write('    95%CI = [{0:.3f}, {1:.3f}]\n'.format(sgmB_p025, sgmB_p975))
txtout.write('    Q1 = {0:.3f},    Median = {1:.3f},    Q3 = {2:.3f}\n'.format(
                    sgmB_p25, sgmB_p50, sgmB_p75))

plt.title('標準偏差の事後分布', fontsize = 20)
sb.kdeplot(fit['sgmA'], label = VarNameA)
sb.kdeplot(fit['sgmB'], label = VarNameB)
plt.xlabel('標準偏差', fontsize = 14)
plt.savefig(foutnm[:-4] + '_sgmAB.png')
plt.show()

#
#       サンプル値の差の事後予測
#
diff = []
for ma, mb, sgma, sgmb in zip(fit['muA'], fit['muB'], fit['sgmA'], fit['sgmB']):
    va = ss.norm.rvs(ma, sgma)
    vb = ss.norm.rvs(mb, sgmb)
    diff.append(va - vb)

diff = np.array(diff)

p_pos_diff = (diff > 0.0).sum() / len(diff)
sb.kdeplot(diff)
plt.title(('サンプル値の差の事後予測 {0} - {1}'.format(VarNameA, VarNameB)) +
          '\n確率（差＞0）= {0:.3f}'.format(p_pos_diff), fontsize = 16)
plt.xlabel('{0}サンプル値 - {1}サンプル値'.format(VarNameA, VarNameB), fontsize = 14)
plt.yticks([])
plt.savefig(foutnm[:-4] + '_postD.png')
plt.show()

txtout.close()
print('\n出力ファイル{}が保存されました。\n'.format(txtoutnm))
